from pathlib import Path

from pathlib import Path
from django.conf import settings
from importlib import import_module
import logging

logger = logging.getLogger(__name__)


class AutodiscoverApps:
    def __init__(self, glob, exclude=[]) -> None:
        self.apps = settings.INSTALLED_APPS
        self.base_path = settings.BASE_DIR
        self.glob = glob
        self.app_paths = []

    def __normalize_module_name(self, module):
        return ".".join(module.parts).replace(".py", "")

    def __call__(self):
        for app in self.apps:
            try:
                paths = import_module(
                    app
                ).__path__  # Path can be actually in multiple directors so is a list
            except AttributeError as e:
                logger.error(f"Balder couldn't import app {app}: {e}")
                continue  # We continue here and skip the rest

            for path in paths:
                logger.debug(f"Trying to import {path}")
                p = Path(path)
                matched_files = list(p.glob(self.glob))
                for file in matched_files:
                    module_name = self.__normalize_module_name(
                        file.relative_to(self.base_path)
                    )
                    print(module_name)
                    import_module(module_name)


autodiscover = AutodiscoverApps(glob="**/balder.py")



from enum import Enum
from typing import Type
from django.db.models.enums import TextChoices
import graphene

class InputEnum:

    @staticmethod
    def from_choices(enum_class: Type[TextChoices], description=None):
        description = description or enum_class.__doc__

        def des(v):
            return enum_class[v.name].label if v is not None else description

        return graphene.Enum(f"{enum_class.__name__}Input", [(tag.name, tag.value) for tag in enum_class], description= des)




import django_filters
from django import forms
from balder.enum import InputEnum
from graphene_django.forms.converter import convert_form_field
import graphene


class EnumMultiChoiceField(forms.MultipleChoiceField):

    def __init__(self, *args, choices=None, type=None, **kwargs) -> None:
        
        self.__choices = choices
        self.__type = type
        self.__choices = [ (str(value.value), str(value.value)) for key, value in self.__type._meta.enum.__members__.items()]
        # casting here is weird as we will get the actual value from our graphene type again

        super().__init__(*args, choices=self.__choices, **kwargs)


    @property
    def overwritten_type(self):

        def construct(*args, **kwargs):
            return graphene.List(self.__type, **kwargs)

        if self.__type: return construct



@convert_form_field.register(EnumMultiChoiceField)
def convert_form_field_to_string_list(field):
    return field.overwritten_type(required=field.required)

class MultiEnumFilter(django_filters.MultipleChoiceFilter):
    field_class = EnumMultiChoiceField





from graphene_file_upload.scalars import Upload
from typing import Callable, List, Union
import graphene
from django.core.validators import validate_email


Validator = Callable[[any], any]


def createValidScalar(
    scalar_name, validator: Validator
) -> Callable[[graphene.String], None]:
    def validate(value: str) -> None:
        return validator(value)

    return type(scalar_name, (graphene.String,), {"serialize": staticmethod(validate)})


class Email(graphene.String):
    @staticmethod
    def serialize(self, value):  #
        validate_email(value=value)
        return str(value)

    @staticmethod
    def parse_literal(node):
        return node.value

    @staticmethod
    def parse_value(value):
        return value


class ImageFile(Upload):
    """An Image File (png, jpg)

    [type]: [description]
    """

    pass



from django.db.models import Model
from django.core import serializers


class PayloadSerializer:

    @classmethod
    def model_serializer(cls, unpacked):
        return serializers.serialize("json", unpacked, fields=('id'))

    @classmethod
    def model_deserializer(cls, packed):
        for obj in serializers.deserialize('json', packed):
            return obj.object
    

    @classmethod
    def pack(cls, unpacked):
        if isinstance(unpacked, Model):
            return {"type": "model", "payload": cls.model_serializer([unpacked])}
        if isinstance(unpacked, dict):
            return {"type": "dict", "payload": unpacked}
        
        return {"type": "scalar", "payload": unpacked}



    @classmethod
    def unpack(cls, packed, basemodel=None):
        type = packed["type"]
        payload = packed["payload"]
        if type == "model": 
            return cls.model_deserializer(payload)
        if type == "dict":
            return payload
        if type == "pydantic":
            assert basemodel is not None, "Please provide a basemodel in your unpack"
            return basemodel(**payload)
        if type == "scalar":
            return payload
        
        raise NotImplementedError(f"Got unknown type {type}")
